/*
 *	SU18 CMPT433 - TeamChess (Joe Xu)
 *	Ported library for the Adafruit RA8875 Driver board for TFT Displays
 *	Heavily based on the code found at:
 *  https://github.com/adafruit/Adafruit_RA8875/blob/master/Adafruit_RA8875.cpp
 *	
 */

#include "spi.h"
#include "lcd.h"
#include "gpio.h"

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>


static int _width = 800;
static int _height = 480;
static struct timespec time_delay;

int lcdInit(){

	/* GPIO INIT */
	gpioInit(RST_PIN, GPIO_OUT, 0);

	//Power cycle LCD
	gpio_write(RST_PIN, 1);
	delay(10);
	gpio_write(RST_PIN, 0);
	delay(10);
	gpio_write(RST_PIN, 1);
	delay(500);

	gpioInit(INT_PIN, GPIO_IN, 0);
	delay(30);


	/* SPI INIT */
	//Reading STSR and wait for a 0x75 reply
	uint8_t reply = spi_read(0);

	if(reply != 0x75){
		
		printf("Initialize failed, wrong reply, check SPI setting\n");
		exit(EXIT_FAILURE);
	}	

	// (_size == RA8875_800x480)
	uint8_t pixclk          = RA8875_PCSR_PDATL | RA8875_PCSR_2CLK;
    uint8_t hsync_nondisp   = 26;
    uint8_t hsync_start     = 32;
    uint8_t hsync_pw        = 96;
    uint8_t hsync_finetune  = 0;
    uint16_t vsync_nondisp  = 32;
    uint16_t vsync_start    = 23;
    uint8_t vsync_pw        = 2;

	spi_write(RA8875_PCSR, pixclk);
	delay(1);

	/* Horizontal settings registers */
	spi_write(RA8875_HDWR, (_width / 8) - 1);                          // H width: (HDWR + 1) * 8 = 480
	spi_write(RA8875_HNDFTR, RA8875_HNDFTR_DE_HIGH + hsync_finetune);
	spi_write(RA8875_HNDR, (hsync_nondisp - hsync_finetune - 2)/8);    // H non-display: HNDR * 8 + HNDFTR + 2 = 10
	spi_write(RA8875_HSTR, hsync_start/8 - 1);                         // Hsync start: (HSTR + 1)*8 
	spi_write(RA8875_HPWR, RA8875_HPWR_LOW + (hsync_pw/8 - 1));        // HSync pulse width = (HPWR+1) * 8

	/* Vertical settings registers */
	spi_write(RA8875_VDHR0, (uint16_t)(_height - 1) & 0xFF);
	spi_write(RA8875_VDHR1, (uint16_t)(_height - 1) >> 8);
	spi_write(RA8875_VNDR0, vsync_nondisp-1);                          // V non-display period = VNDR + 1
	spi_write(RA8875_VNDR1, vsync_nondisp >> 8);
	spi_write(RA8875_VSTR0, vsync_start-1);                            // Vsync start position = VSTR + 1
	spi_write(RA8875_VSTR1, vsync_start >> 8);
	spi_write(RA8875_VPWR, RA8875_VPWR_LOW + vsync_pw - 1);            // Vsync pulse width = VPWR + 1

	/* Set active window X */
	spi_write(RA8875_HSAW0, 0);                                        // horizontal start point
	spi_write(RA8875_HSAW1, 0);
	spi_write(RA8875_HEAW0, (uint16_t)(_width - 1) & 0xFF);            // horizontal end point
	spi_write(RA8875_HEAW1, (uint16_t)(_width - 1) >> 8);

	/* Set active window Y */
	spi_write(RA8875_VSAW0, 0);                                        // vertical start point
	spi_write(RA8875_VSAW1, 0);  
	spi_write(RA8875_VEAW0, (uint16_t)(_height - 1) & 0xFF);           // horizontal end point
	spi_write(RA8875_VEAW1, (uint16_t)(_height - 1) >> 8);

	/* Clear the entire window */
	spi_write(RA8875_MCLR, RA8875_MCLR_START | RA8875_MCLR_FULL);
	delay(500); 

  	// Display on
  	spi_write(RA8875_PWRR, RA8875_PWRR_NORMAL | RA8875_PWRR_DISPON);

  	// Enable GPIOX
	spi_write(RA8875_GPIOX, 1);

	// PWM Config
	spi_write(RA8875_P1CR, RA8875_P1CR_ENABLE | (RA8875_PWM_CLK_DIV1024 & 0xF));
	spi_write(0x8c, RA8875_P2CR_ENABLE | (RA8875_PWM_CLK_DIV1024 & 0xF));

	// Brightness
	setBrightness(255);

	// Enable touch panel
	touchPanelEnable(1);

	// Enable 2 layers
	spi_write(0x20, spi_read(0x20) | 0x80);

	printf("LCD initialization success\n");

	return 0;
}

void setBrightness(uint8_t bright){

	spi_write(RA8875_P1DCR, bright);
	spi_write(RA8875_P2DCR, bright);
}

void touchPanelEnable(int i){
	uint8_t   adcClk = (uint8_t) RA8875_TPCR0_ADCCLK_DIV16;
	if (i == 1) {
		spi_write(RA8875_TPCR0, RA8875_TPCR0_ENABLE        |
		                       RA8875_TPCR0_WAIT_4096CLK  |
		                       RA8875_TPCR0_WAKEENABLE   |
		                       adcClk); 

		spi_write(RA8875_TPCR1, RA8875_TPCR1_AUTO    |
		                       RA8875_TPCR1_DEBOUNCE);

		uint8_t ret = spi_read(RA8875_INTC1);
		spi_write(RA8875_INTC1, ret | RA8875_INTC1_TP);

	} else {
		uint8_t ret1 = spi_read(RA8875_INTC1);
		spi_write(RA8875_INTC1, ret1 & (~RA8875_INTC1_TP) );
		spi_write(RA8875_TPCR0, RA8875_TPCR0_DISABLE);
	}
}

void delay(int miliseconds){
	time_delay.tv_sec = 0;
	time_delay.tv_nsec = miliseconds * 1000000L;

	nanosleep(&time_delay, NULL);
}
